
// $Id:$
// package com.xmlglobal.ebxml.cc;
import java.io.*;
import java.util.*;
import org.jdom.*; // build 7.  < 6 may not work.
import org.jdom.input.SAXBuilder;
import org.jdom.output.*;

/******************************************
 *
 *  You will need to place crimson.jar, jdom.jar
 *  and jaxp.jar into your classpath
 *  (c) 2001 Duane Nickull, Matt Mackenzie - XML Global Technologies, Inc.
 *  All rights reserved.  You may use, distribute or modify this code
 *  freely as long as this message remains intact.
 *
 *  Changed 25.09.01 by D. Nickull to work with updated CC model in CVS
 */

public class AssemblyUtils {
  static Namespace rngNs = Namespace.getNamespace("http://relaxng.org/ns/structure/0.9");
  public AssemblyUtils() { }

  public static Element extractAssembly(Element cc, Element cr) throws Exception {

    // Grabs the ID from the Context Rules message.
    String contextId = cr.getAttributeValue("ID");

    // Create a list of Context Assemblies in memory.
    List asms = cc.getChildren("Assembly");

    Element result = null; // initialize

    // Iterate through the List
    for (int i = 0; i < asms.size(); i++) {
      Element asm = (Element)asms.get(i);
      //System.out.println(asm);
      //Grab each Element
      List ctxts = asm.getChild("ExtendedDetails").getChild("ContextGroups").getChildren("Context");

      // Since there can be more than one Context, we have to test them all
      for (int ii = 0; ii < ctxts.size(); ii++) {
        Element ctxt = (Element)ctxts.get(ii);

        // Grab the ID attributes value
        String ctxtId = ctxt.getAttributeValue("ID");

        // Test it against the Context Rules value.
        // Beware of funny results with evaluating null values!
         if (ctxtId !=null && ctxtId.equals(contextId)) {

          // If it matches, grab the branch of rng grammar or element
          Element rng = asm.getChild("ExtendedDetails").getChild("grammar", rngNs);
          if (rng == null) rng = asm.getChild("ExtendedDetails").getChild("element", rngNs);

          // Test to see if we found anything.
          if (rng == null) throw new Exception("No RNG schema found in assembly");

          // Return branch as fragment
          return rng.detach();
        }

      }

    }
    throw new Exception("Evidently, something is wrong with your XML.");
  }

  public static void main(String[] args) {
    if (args.length < 2) {
      System.out.println(
        "Usage: java " + AssemblyUtils.class.getName() +
        "<CoreComponent.xml> <ContextRules.xml>");
      System.exit(-1);
    }
    try {
      Element coreComp = new SAXBuilder().build(args[0]).getRootElement();
      Element contRule = new SAXBuilder().build(args[1]).getRootElement();

      // test the Context Rules document. Not needed if validated at parse time.
      if (!contRule.getName().equals("ContextRules")) {
        System.err.println("The input does not appear to be a valid context rules document.");
        System.exit(-1);
      }

      Element target = null;

      // If we call this code from another program we feed it two args
      // and get the fragment returned.
      try {
        target = AssemblyUtils.extractAssembly(coreComp, contRule);
        XMLOutputter xo = new XMLOutputter("  ", true);
        xo.output(target, System.out);
      } catch (Exception e) {
        System.err.println("Error extracting assembly: " + e.getMessage());
        System.exit(-1);
      }
    } catch (Exception e) {
      System.err.println("XML Parser error: " + e.getMessage());
    }
  }
}


